@@ -1,12 +1,16 @@
 /* (from) $XConsortium: SimpleMenu.c,v 1.41 92/09/10 16:25:07 converse Exp $ */
 
 /*
- * Modifications Copyright 1995 by Paul Mattes.
- *   Permission to use, copy, modify, and distribute this software and its
- *   documentation for any purpose and without fee is hereby granted,
- *   provided that the above copyright notice appear in all copies and that
- *   both that copyright notice and this permission notice appear in
- *   supporting documentation.
+ * Modifications Copyright 1995, 1999 by Paul Mattes.
+ *  Permission to use, copy, modify, and distribute this software and its
+ *  documentation for any purpose and without fee is hereby granted,
+ *  provided that the above copyright notice appear in all copies and that
+ *  both that copyright notice and this permission notice appear in
+ *  supporting documentation.
+ *
+ * x3270 is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the file LICENSE for more details.
  *
  * Copyright 1989 Massachusetts Institute of Technology
  *
@@ -39,6 +43,9 @@
  *          kit@expo.lcs.mit.edu
  */
 
+#include "globals.h"
+#if defined(X3270_MENUS) /*[*/
+
 #include <stdio.h>
 #include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
@@ -111,29 +118,45 @@ static char defaultTranslations[] =
  * Semi Public function definitions. 
  */
 
-static void Redisplay(), Realize(), Resize(), ChangeManaged();
-static void Initialize(), ClassInitialize(), ClassPartInitialize();
-static Boolean SetValues(), SetValuesHook();
-static XtGeometryResult GeometryManager();
+static void Redisplay(Widget, XEvent *, Region);
+static void Realize(Widget, XtValueMask *, XSetWindowAttributes *);
+static void ChangeManaged(Widget);
+static void Resize(Widget);
+static void Initialize(Widget, Widget, ArgList, Cardinal *);
+static void ClassInitialize(void);
+static void ClassPartInitialize(WidgetClass);
+static Boolean SetValues(Widget, Widget, Widget, ArgList, Cardinal *);
+static Boolean SetValuesHook(Widget, ArgList, Cardinal *);
+static XtGeometryResult GeometryManager(Widget, XtWidgetGeometry *,
+	XtWidgetGeometry *);
 
 /*
  * Action Routine Definitions
  */
 
-static void Highlight(), LeftWindow(), SaveUnhighlight(), PositionMenuAction();
+static void PositionMenuAction(Widget, XEvent *, String *, Cardinal *);
+static void SaveUnhighlight(Widget, XEvent *, String *, Cardinal *);
+static void LeftWindow(Widget, XEvent *, String *, Cardinal *);
+static void Highlight(Widget, XEvent *, String *, Cardinal *);
 
 /* 
  * Private Function Definitions.
  */
 
 static void Unhighlight(), Notify();
-static void MakeSetValuesRequest(), CreateLabel(), Layout();
-static void AddPositionAction(), PositionMenu(), ChangeCursorOnGrab();
-static void ClearParent();
-static Dimension GetMenuWidth(), GetMenuHeight();
-static Widget FindMenu();
-static CmeObject GetEventEntry(), GetRightEntry();
-static void MoveMenu();
+static void MakeSetValuesRequest(Widget, Dimension, Dimension);
+static void Layout(Widget, Dimension *, Dimension *);
+static void CreateLabel(Widget);
+static void AddPositionAction(XtAppContext, caddr_t);
+static void ChangeCursorOnGrab(Widget, XtPointer junk, XtPointer);
+static void PositionMenu(Widget, XPoint *);
+static void ClearParent(Widget, XtPointer, XtPointer);
+static Dimension GetMenuWidth(Widget, Widget);
+static Dimension GetMenuHeight(Widget);
+static Widget FindMenu(Widget, String);
+static CmeObject GetEventEntry(Widget, XEvent *);
+static CmeObject GetRightEntry(Widget, XEvent *);
+static void MoveMenu(Widget, Position, Position);
 
 static XtActionsRec actionsList[] =
 {
@@ -216,7 +239,7 @@ WidgetClass complexMenuWidgetClass = (WidgetClass)&complexMenuClassRec;
  */
 
 static void
-ClassInitialize()
+ClassInitialize(void)
 {
   XawInitializeWidgetSet();
   XtAddConverter( XtRString, XtRBackingStore, XmuCvtStringToBackingStore,
@@ -233,8 +256,7 @@ ClassInitialize()
  */
 
 static void
-ClassPartInitialize(wc)
-WidgetClass wc;
+ClassPartInitialize(WidgetClass wc)
 {
     ComplexMenuWidgetClass cmwc = (ComplexMenuWidgetClass) wc;
 
@@ -254,12 +276,9 @@ WidgetClass wc;
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
-Initialize(request, new, args, num_args)
-Widget request, new;
-ArgList args;
-Cardinal *num_args;
+Initialize(Widget request unused, Widget new, ArgList args unused,
+	Cardinal *num_args unused)
 {
   ComplexMenuWidget cmw = (ComplexMenuWidget) new;
 
@@ -311,12 +330,8 @@ Cardinal *num_args;
  *      Returns: none.
  */
 
-/* ARGSUSED */
 static void
-Redisplay(w, event, region)
-Widget w;
-XEvent * event;
-Region region;
+Redisplay(Widget w, XEvent *event unused, Region region)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject * entry;
@@ -359,10 +374,7 @@ Region region;
  */
 
 static void
-Realize(w, mask, attrs)
-Widget w;
-XtValueMask * mask;
-XSetWindowAttributes * attrs;
+Realize(Widget w, XtValueMask *mask, XSetWindowAttributes *attrs)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
 
@@ -387,8 +399,7 @@ XSetWindowAttributes * attrs;
  */
 
 static void
-Resize(w)
-Widget w;
+Resize(Widget w)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject * entry;
@@ -410,12 +421,9 @@ Widget w;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static Boolean
-SetValues(current, request, new, args, num_args)
-Widget current, request, new;
-ArgList args;
-Cardinal *num_args;
+SetValues(Widget current, Widget request unused, Widget new,
+	ArgList args unused, Cardinal *num_args unused)
 {
     ComplexMenuWidget cmw_old = (ComplexMenuWidget) current;
     ComplexMenuWidget cmw_new = (ComplexMenuWidget) new;
@@ -438,7 +446,7 @@ Cardinal *num_args;
 	XDefineCursor(XtDisplay(new),
 		      XtWindow(new), cmw_new->complex_menu.cursor);
     
-    if (cmw_old->complex_menu.label_string !=cmw_new->complex_menu.label_string) 
+    if (cmw_old->complex_menu.label_string !=cmw_new->complex_menu.label_string) { 
 	if (cmw_new->complex_menu.label_string == NULL)         /* Destroy. */
 	    XtDestroyWidget((Widget) cmw_old->complex_menu.label);
 	else if (cmw_old->complex_menu.label_string == NULL)    /* Create. */
@@ -449,6 +457,7 @@ Cardinal *num_args;
 	    XtSetArg(arglist[0], XtNlabel, cmw_new->complex_menu.label_string);
 	    XtSetValues((Widget) cmw_new->complex_menu.label, arglist, ONE);
 	}
+    }
     
     if (cmw_old->complex_menu.label_class != cmw_new->complex_menu.label_class)
 	XtAppWarning(XtWidgetToApplicationContext(new),
@@ -483,10 +492,7 @@ Cardinal *num_args;
  */
 
 static Boolean
-SetValuesHook(w, arglist, num_args)
-Widget w;
-ArgList arglist;
-Cardinal *num_args;
+SetValuesHook(Widget w, ArgList arglist, Cardinal *num_args)
 {
     register Cardinal i;
     Dimension width, height;
@@ -521,9 +527,7 @@ Cardinal *num_args;
  */
 
 static XtGeometryResult
-GeometryManager(w, request, reply)
-Widget w;
-XtWidgetGeometry * request, * reply;
+GeometryManager(Widget w, XtWidgetGeometry *request, XtWidgetGeometry *reply)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) XtParent(w);
     CmeObject entry = (CmeObject) w;
@@ -591,8 +595,7 @@ XtWidgetGeometry * request, * reply;
  */
 
 static void
-ChangeManaged(w)
-Widget w;
+ChangeManaged(Widget w)
 {
     Layout(w, (Dimension *)NULL, (Dimension *)NULL);
 }
@@ -615,13 +618,9 @@ Widget w;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static void
-PositionMenuAction(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+PositionMenuAction(Widget w, XEvent *event, String *params,
+	Cardinal *num_params)
 { 
   Widget menu;
   XPoint loc;
@@ -681,13 +680,9 @@ Cardinal * num_params;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static void
-Unhighlight(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+Unhighlight(Widget w, XEvent *event unused, String *params unused,
+	Cardinal *num_params unused)
 { 
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry = cmw->complex_menu.entry_set;
@@ -700,13 +695,9 @@ Cardinal * num_params;
     (class->cme_class.unhighlight) ( (Widget) entry);
 }
 
-/* ARGSUSED */
 static void
-SaveUnhighlight(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+SaveUnhighlight(Widget w, XEvent *event unused, String *params unused,
+	Cardinal *num_params unused)
 { 
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry = cmw->complex_menu.entry_set;
@@ -728,13 +719,9 @@ Cardinal * num_params;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static void
-LeftWindow(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+LeftWindow(Widget w, XEvent *event, String *params unused,
+	Cardinal *num_params unused)
 { 
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry = cmw->complex_menu.entry_set;
@@ -762,13 +749,8 @@ Cardinal * num_params;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static void
-Highlight(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+Highlight(Widget w, XEvent *event, String *params, Cardinal *num_params)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry;
@@ -816,13 +798,9 @@ Cardinal * num_params;
  *      Returns: none
  */
 
-/* ARGSUSED */
 static void
-Notify(w, event, params, num_params)
-Widget w;
-XEvent * event;
-String * params;
-Cardinal * num_params;
+Notify(Widget w, XEvent *event unused, String *params unused,
+	Cardinal *num_params unused)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry = cmw->complex_menu.entry_set;
@@ -869,12 +847,7 @@ Cardinal * num_params;
  */
 
 void
-#if NeedFunctionPrototypes
 XawComplexMenuAddGlobalActions(XtAppContext app_con)
-#else
-XawComplexMenuAddGlobalActions(app_con)
-XtAppContext app_con;
-#endif
 {
     XtInitializeWidgetClass(complexMenuWidgetClass);
     XmuCallInitializers( app_con );
@@ -888,12 +861,7 @@ XtAppContext app_con;
  */
 
 Widget
-#if NeedFunctionPrototypes
 XawComplexMenuGetActiveEntry(Widget w)
-#else
-XawComplexMenuGetActiveEntry(w)
-Widget w;
-#endif
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
 
@@ -907,12 +875,7 @@ Widget w;
  */
 
 void
-#if NeedFunctionPrototypes
 XawComplexMenuClearActiveEntry(Widget w)
-#else
-XawComplexMenuClearActiveEntry(w)
-Widget w;
-#endif
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
 
@@ -936,8 +899,7 @@ Widget w;
  */
 
 static void
-CreateLabel(w)
-Widget w;
+CreateLabel(Widget w)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     register Widget * child, * next_child;
@@ -990,9 +952,7 @@ Widget w;
  */
 
 static void
-Layout(w, width_ret, height_ret)
-Widget w;
-Dimension *width_ret, *height_ret;
+Layout(Widget w, Dimension *width_ret, Dimension *height_ret)
 {
     CmeObject current_entry, *entry;
     ComplexMenuWidget cmw;
@@ -1067,11 +1027,8 @@ Dimension *width_ret, *height_ret;
  *	Returns: none.
  */
 
-/* ARGSUSED */
 static void
-AddPositionAction(app_con, data)
-XtAppContext app_con;
-caddr_t data;
+AddPositionAction(XtAppContext app_con, caddr_t data unused)
 {
     static XtActionsRec pos_action[] = {
         { "XawPositionComplexMenu", PositionMenuAction },
@@ -1088,9 +1045,7 @@ caddr_t data;
  */
 
 static Widget 
-FindMenu(widget, name)
-Widget widget;
-String name;
+FindMenu(Widget widget, String name)
 {
     register Widget w, menu;
     
@@ -1108,9 +1063,7 @@ String name;
  */
 
 static void
-PositionMenu(w, location)
-Widget w;
-XPoint * location;
+PositionMenu(Widget w, XPoint *location)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject entry;
@@ -1162,9 +1115,7 @@ XPoint * location;
  */
 
 static void
-MoveMenu(w, x, y)
-Widget w;
-Position x, y;
+MoveMenu(Widget w, Position x, Position y)
 {
     Arg arglist[2];
     Cardinal num_args = 0;
@@ -1204,11 +1155,8 @@ Position x, y;
  *	Returns: None.
  */
 
-/* ARGSUSED */
 static void
-ChangeCursorOnGrab(w, junk, garbage)
-Widget w;
-XtPointer junk, garbage;
+ChangeCursorOnGrab(Widget w, XtPointer junk unused, XtPointer garbage unused)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     
@@ -1229,11 +1177,8 @@ XtPointer junk, garbage;
 			     XtLastTimestampProcessed(XtDisplay(w)));
 }
 
-/*ARGSUSED*/
 static void
-ClearParent(w, junk, garbage)
-Widget w;
-XtPointer junk, garbage;
+ClearParent(Widget w, XtPointer junk unused, XtPointer garbage unused)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     
@@ -1253,9 +1198,7 @@ XtPointer junk, garbage;
  */
 
 static void
-MakeSetValuesRequest(w, width, height)
-Widget w;
-Dimension width, height;
+MakeSetValuesRequest(Widget w, Dimension width, Dimension height)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     Arg arglist[2];
@@ -1281,8 +1224,7 @@ Dimension width, height;
  */
 
 static Dimension
-GetMenuWidth(w, w_ent)
-Widget w, w_ent;
+GetMenuWidth(Widget w, Widget w_ent)
 {
     CmeObject cur_entry = (CmeObject) w_ent;
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
@@ -1322,8 +1264,7 @@ Widget w, w_ent;
  */
 
 static Dimension
-GetMenuHeight(w)
-Widget w;
+GetMenuHeight(Widget w)
 {
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject * entry;
@@ -1352,11 +1293,9 @@ Widget w;
  */
 
 static CmeObject
-GetEventEntry(w, event)
-Widget w;
-XEvent * event;
+GetEventEntry(Widget w, XEvent *event)
 {
-    Position x_loc, y_loc;
+    Position x_loc = 0, y_loc = 0;
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
     CmeObject * entry;
     
@@ -1389,11 +1328,12 @@ XEvent * event;
 	if (!XtIsManaged ((Widget) *entry)) continue;
 
 	if ( ((*entry)->rectangle.y < y_loc) &&
-	    ((*entry)->rectangle.y + (int) (*entry)->rectangle.height > y_loc) )
+            ((*entry)->rectangle.y + (int) (*entry)->rectangle.height > y_loc) ) {
 	    if ( *entry == cmw->complex_menu.label )
 		return(NULL);	/* cannot select the label. */
 	    else
 		return(*entry);
+        }
     }
     
     return(NULL);
@@ -1408,9 +1348,7 @@ XEvent * event;
  */
 
 static CmeObject
-GetRightEntry(w, event)
-Widget w;
-XEvent * event;
+GetRightEntry(Widget w, XEvent *event)
 {
     Position x_loc, y_loc;
     ComplexMenuWidget cmw = (ComplexMenuWidget) w;
@@ -1427,12 +1365,15 @@ XEvent * event;
 	if (!XtIsManaged ((Widget) *entry)) continue;
 
 	if ( ((*entry)->rectangle.y < y_loc) &&
-	    ((*entry)->rectangle.y + (int) (*entry)->rectangle.height > y_loc) )
+            ((*entry)->rectangle.y + (int) (*entry)->rectangle.height > y_loc) ) {
 	    if ( *entry == cmw->complex_menu.label )
 		return(NULL);	/* cannot select the label. */
 	    else
 		return(*entry);
+        }
     }
 
     return(NULL);
 }
+
+#endif /*]*/
