@@ -1,21 +1,28 @@
 /*
- * Modifications Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996, 1999,
+ *   2000, 2001, 2002 by Paul Mattes.
  * Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
  *  both that copyright notice and this permission notice appear in
  *  supporting documentation.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
  *	appres.h
- *		Application resource definitions for x3270.
+ *		Application resource definitions for x3270, c3270, s3270 and
+ *		tcl3270.
  */
 
 /* Toggles */
 
-enum toggle_type { TT_INITIAL, TT_TOGGLE, TT_FINAL };
+enum toggle_type { TT_INITIAL, TT_INTERACTIVE, TT_ACTION, TT_FINAL };
 struct toggle {
 	Boolean value;		/* toggle value */
 	Boolean changed;	/* has the value changed since init */
@@ -49,7 +56,11 @@ struct toggle {
 #define MARGINED_PASTE	11
 #define RECTANGLE_SELECT 12
 
-#define N_TOGGLES	13
+#if defined(X3270_DISPLAY) /*[*/
+#define CROSSHAIR	13
+#endif /*]*/
+
+#define N_TOGGLES	14
 
 #define toggled(ix)		(appres.toggle[ix].value)
 #define toggle_toggle(t) \
@@ -88,6 +99,8 @@ typedef struct {
 #endif /*]*/
 #if defined(C3270) /*[*/
 	Boolean all_bold_on;
+	Boolean	curses_keypad;
+	Boolean cbreak_mode;
 #endif /*]*/
 	Boolean	apl_mode;
 	Boolean scripted;
@@ -107,6 +120,7 @@ typedef struct {
 #if defined(X3270_DISPLAY) || defined(C3270) /*[*/
 	char	*key_map;
 	char	*compose_map;
+	char	*printer_lu;
 #endif /*]*/
 #if defined(X3270_DISPLAY) /*[*/
 	char	*efontname;
@@ -131,6 +145,8 @@ typedef struct {
 #if defined(C3270) /*[*/
 	char	*meta_escape;
 	char	*all_bold;
+	char	*altscreen;
+	char	*defscreen;
 #endif /*]*/
 	char	*conf_dir;
 	char	*model;
@@ -153,6 +169,9 @@ typedef struct {
 	char	*ft_command;
 #endif /*]*/
 	char	*connectfile_name;
+	char	*idle_command;
+	Boolean idle_command_enabled;
+	char	*idle_timeout;
 
 	/* Toggles */
 	struct toggle toggle[N_TOGGLES];
