@@ -1,5 +1,6 @@
 /*
- * Modifications Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996, 1999,
+ *  2000, 2001, 2002 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -11,6 +12,11 @@
  *  All Rights Reserved.  GTRC hereby grants public use of this software.
  *  Derivative works based on this software must incorporate this copyright
  *  notice.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
@@ -413,11 +419,11 @@ ctlr_erase(Boolean alt)
 	if (alt == screen_alt)
 		return;
 
-	screen_disp();
+	screen_disp(True);
 
 	if (alt) {
 		/* Going from 24x80 to maximum. */
-		screen_disp();
+		screen_disp(False);
 		ROWS = maxROWS;
 		COLS = maxCOLS;
 	} else {
@@ -425,7 +431,7 @@ ctlr_erase(Boolean alt)
 		if (maxROWS > 24 || maxCOLS > 80) {
 			if (*debugging_font) {
 				ctlr_blanks();
-				screen_disp();
+				screen_disp(False);
 			}
 			ROWS = 24;
 			COLS = 80;
@@ -1861,7 +1867,7 @@ ctlr_scroll(void)
 	/* Synchronize pending changes prior to this. */
 	obscured = screen_obscured();
 	if (!obscured && screen_changed)
-		screen_disp();
+		screen_disp(False);
 
 	/* Move screen_buf and ea_buf. */
 	(void) memmove(&screen_buf[0], &screen_buf[COLS], qty);
@@ -1958,7 +1964,7 @@ ctlr_shrink(void)
 	    *debugging_font ? CG_space : CG_null,
 	    ROWS*COLS);
 	ALL_CHANGED;
-	screen_disp();
+	screen_disp(False);
 }
 
 
@@ -1970,6 +1976,7 @@ ctlr_shrink(void)
  */
 static struct timeval t_start;
 static Boolean ticking = False;
+static Boolean mticking = False;
 static unsigned long tick_id;
 static struct timeval t_want;
 
@@ -1999,13 +2006,15 @@ keep_ticking(void)
 void
 ticking_start(Boolean anyway)
 {
+	(void) gettimeofday(&t_start, (struct timezone *) 0);
+	mticking = True;
+
 	if (!toggled(SHOW_TIMING) && !anyway)
 		return;
 	status_untiming();
 	if (ticking)
 		RemoveTimeOut(tick_id);
 	ticking = True;
-	(void) gettimeofday(&t_start, (struct timezone *) 0);
 	tick_id = AddTimeOut(1000, keep_ticking);
 	t_want = t_start;
 }
@@ -2015,10 +2024,17 @@ ticking_stop(void)
 {
 	struct timeval t1;
 
+	(void) gettimeofday(&t1, (struct timezone *) 0);
+	if (mticking) {
+		sms_accumulate_time(&t_start, &t1);
+		mticking = False;
+	} else {
+		return;
+	}
+
 	if (!ticking)
 		return;
 	RemoveTimeOut(tick_id);
-	(void) gettimeofday(&t1, (struct timezone *) 0);
 	ticking = False;
 	status_timing(&t_start, &t1);
 }
