@@ -1,5 +1,6 @@
 /*
- * Modifications Copyright 1993, 1994, 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
+ * Modifications Copyright 1993, 1994, 1995, 1996, 1999,
+ *  2000, 2001, 2002 by Paul Mattes.
  * Original X11 Port Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
@@ -11,6 +12,11 @@
  *  All Rights Reserved.  GTRC hereby grants public use of this software.
  *  Derivative works based on this software must incorporate this copyright
  *  notice.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
@@ -42,6 +48,7 @@
 #include "ctlrc.h"
 #include "ftc.h"
 #include "hostc.h"
+#include "idlec.h"
 #include "keymapc.h"
 #include "keypadc.h"
 #include "kybdc.h"
@@ -401,10 +408,11 @@ key_AID(unsigned char aid_code)
 		insert_mode(False);
 		kybdlock_set(KL_OIA_TWAIT | KL_OIA_LOCKED, "key_AID");
 	}
+	reset_idle_timer();
 	aid = aid_code;
 	ctlr_read_modified(aid, False);
+	ticking_start(False);
 	if (!IN_SSCP) {
-		ticking_start(False);
 		status_ctlr_done();
 	}
 }
