@@ -1,15 +1,20 @@
 /*
- * Copyright 1995, 1996, 1999, 2000, 2001 by Paul Mattes.
+ * Copyright 1995, 1996, 1999, 2000, 2001, 2002 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
  *  both that copyright notice and this permission notice appear in
  *  supporting documentation.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
  *	resources.h
- *		x3270 resource and option names.
+ *		x3270/c3270/s3270/tcl3270 resource and option names.
  */
 
 /* Resources. */
@@ -18,6 +23,7 @@
 #define ResAllBold		"allBold"
 #define ResAllowResize		"allowResize"
 #define ResAltCursor		"altCursor"
+#define ResAltScreen		"altScreen"
 #define ResAplFont		"aplFont"
 #define ResAplMode		"aplMode"
 #define ResAssocCommand		"printer.assocCommandLine"
@@ -25,6 +31,7 @@
 #define ResBellVolume		"bellVolume"
 #define ResBlankFill		"blankFill"
 #define ResBoldColor		"boldColor"
+#define ResCbreak		"cbreak"
 #define ResCharClass		"charClass"
 #define ResCharset		"charset"
 #define ResCharsetList		"charsetList"
@@ -35,11 +42,14 @@
 #define ResComposeMap		"composeMap"
 #define ResConfDir		"confDir"
 #define ResConnectFileName	"connectFileName"
+#define ResCrosshair		"crosshair"
+#define ResCursesKeypad		"cursesKeypad"
 #define ResCursorBlink		"cursorBlink"
 #define ResCursorColor		"cursorColor"
 #define ResCursorPos		"cursorPos"
 #define ResDebugFont		"debugFont"
 #define ResDebugTracing		"debugTracing"
+#define ResDefScreen		"defScreen"
 #define ResDisconnectClear	"disconnectClear"
 #define ResDisplayCharset	"displayCharset"
 #define ResDoConfirms		"doConfirms"
@@ -56,6 +66,9 @@
 #define ResIconFont		"iconFont"
 #define ResIconLabelFont	"iconLabelFont"
 #define ResIcrnl		"icrnl"
+#define ResIdleCommand		"idleCommand"
+#define ResIdleCommandEnabled	"idleCommandEnabled"
+#define ResIdleTimeout		"idleTimeout"
 #define ResInlcr		"inlcr"
 #define ResInputColor		"inputColor"
 #define ResIntr			"intr"
@@ -90,6 +103,7 @@
 #define ResOversize		"oversize"
 #define ResPort			"port"
 #define ResPrinterCommand	"printer.command"
+#define ResPrinterLu		"printerLu"
 #define ResQuit			"quit"
 #define ResReconnect		"reconnect"
 #define ResRectangleSelect	"rectangleSelect"
@@ -119,6 +133,7 @@
 /* Dotted resource names. */
 #define DotActiveIcon		"." ResActiveIcon
 #define DotAplMode		"." ResAplMode
+#define DotCbreak		"." ResCbreak
 #define DotCharClass		"." ResCharClass
 #define DotCharset		"." ResCharset
 #define DotColorScheme		"." ResColorScheme
@@ -133,6 +148,7 @@
 #define DotOnce			"." ResOnce
 #define DotOversize		"." ResOversize
 #define DotPort			"." ResPort
+#define DotPrinterLu		"." ResPrinterLu
 #define DotReconnect		"." ResReconnect
 #define DotSaveLines		"." ResSaveLines
 #define DotScripted		"." ResScripted
@@ -153,6 +169,7 @@
 #define ClsBellVolume		"BellVolume"
 #define ClsBlankFill		"BlankFill"
 #define ClsBoldColor		"BoldColor"
+#define ClsCbreak		"Cbreak"
 #define ClsCharClass		"CharClass"
 #define ClsCharset		"Charset"
 #define ClsColor8		"Color8"
@@ -161,6 +178,7 @@
 #define ClsComposeMap		"ComposeMap"
 #define ClsConfDir		"ConfDir"
 #define ClsConnectFileName	"ConnectFileName"
+#define ClsCrosshair		"Crosshair"
 #define ClsCursorBlink		"CursorBlink"
 #define ClsCursorColor		"CursorColor"
 #define ClsCursorPos		"CursorPos"
@@ -180,6 +198,9 @@
 #define ClsIconFont		"IconFont"
 #define ClsIconLabelFont	"IconLabelFont"
 #define ClsIcrnl		"Icrnl"
+#define ClsIdleCommand		"IdleCommand"
+#define ClsIdleCommandEnabled	"IdleCommandEnabled"
+#define ClsIdleTimeout		"IdleTimeout"
 #define ClsInlcr		"Inlcr"
 #define ClsInputColor		"InputColor"
 #define ClsIntr			"Intr"
@@ -212,6 +233,7 @@
 #define ClsOnlcr		"Onlcr"
 #define ClsOversize		"Oversize"
 #define ClsPort			"Port"
+#define ClsPrinterLu		"PrinterLu"
 #define ClsQuit			"Quit"
 #define ClsReconnect		"Reconnect"
 #define ClsRectangleSelect	"RectangleSelect"
@@ -240,11 +262,14 @@
 /* Options. */
 #define OptActiveIcon		"-activeicon"
 #define OptAllBold		"-allbold"
+#define OptAltScreen		"-altscreen"
 #define OptAplMode		"-apl"
+#define OptCbreak		"-cbreak"
 #define OptCharClass		"-cc"
 #define OptCharset		"-charset"
 #define OptClear		"-clear"
 #define OptColorScheme		"-scheme"
+#define OptDefScreen		"-defscreen"
 #define OptDsTrace		"-trace"
 #define OptEmulatorFont		"-efont"
 #define OptExtended		"-extended"
@@ -262,8 +287,10 @@
 #define OptOnce			"-once"
 #define OptOversize		"-oversize"
 #define OptPort			"-port"
+#define OptPrinterLu		"-printerlu"
 #define OptReconnect		"-reconnect"
 #define OptSaveLines		"-sl"
+#define OptSecure		"-secure"
 #define OptScripted		"-script"
 #define OptScrollBar		"-sb"
 #define OptSet			"-set"
