@@ -1,10 +1,14 @@
 /*
- * Copyright 1993, 1994, 1995, 1999, 2000 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001, 2002 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
  *  both that copyright notice and this permission notice appear in
  *  supporting documentation.
+ *
+ * x3270 is distributed in the hope that it will be useful, but WITHOUT ANY
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ * FOR A PARTICULAR PURPOSE.  See the file LICENSE for more details.
  */
 
 /*
@@ -88,7 +92,7 @@ static int colors3279[SSZ] =  {
  *   M-31	Alternate keymap indication ("K" or blank)
  *   M-30	Reverse input mode indication ("R" or blank)
  *   M-29	Insert mode indication (Special symbol/"I" or blank)
- *   M-28	empty
+ *   M-28	Printer indication ("P" or blank)
  *   M-27	Script indication ("S" or blank)
  *   M-26..M-16	empty
  *   M-15..M-9	command timing (Clock symbol and m:ss, or blank)
@@ -115,8 +119,13 @@ static int colors3279[SSZ] =  {
 
 #define INSERT	(maxCOLS-29)	/* insert mode */
 
+#define PSESS	(maxCOLS-28)	/* printer session */
+
 #define SCRIPT	(maxCOLS-27)	/* script in progress */
 
+#define LU	(maxCOLS-25)	/* LU name */
+#define LUCNT	8
+
 #define T0	(maxCOLS-15)	/* timings */
 #define	TCNT	7
 
@@ -166,6 +175,7 @@ static enum msg {
 	SCROLLED,		/* X Scrolled */
 	MINUS			/* X -f */
 }               oia_msg = DISCONNECTED, saved_msg;
+static char	oia_lu[LUCNT+1];
 static Boolean  msg_is_saved = False;
 static int      n_scrolled = 0;
 static void     (*msg_proc[])(void) = {
@@ -215,6 +225,7 @@ static Boolean  oia_insert = False;
 static Boolean  oia_reverse = False;
 static Boolean  oia_kmap = False;
 static Boolean	oia_script = False;
+static Boolean	oia_printer = False;
 static char    *oia_cursor = (char *) 0;
 static char    *oia_timing = (char *) 0;
 
@@ -260,9 +271,11 @@ static void do_insert(Boolean on);
 static void do_reverse(Boolean on);
 static void do_kmap(Boolean on);
 static void do_script(Boolean on);
+static void do_printer(Boolean on);
 static void do_shift(int state);
 static void do_typeahead(int state);
 static void do_compose(Boolean on, unsigned char c, enum keytype keytype);
+static void do_lu(const char *lu);
 static void do_timing(char *buf);
 static void do_cursor(char *buf);
 
@@ -270,6 +283,7 @@ static void status_connect(Boolean connected);
 static void status_3270_mode(Boolean connected);
 static void status_resolving(Boolean ignored);
 static void status_half_connect(Boolean ignored);
+static void status_printer(Boolean on);
 
 
 /* Initialize the status line */
@@ -296,6 +310,7 @@ status_init(void)
 	register_schange(ST_HALF_CONNECT, status_half_connect);
 	register_schange(ST_CONNECT, status_connect);
 	register_schange(ST_3270_MODE, status_3270_mode);
+	register_schange(ST_PRINTER, status_printer);
 }
 
 /* Reinitialize the status line */
@@ -354,9 +369,11 @@ status_reinit(unsigned cmask)
 	do_reverse(oia_reverse);
 	do_kmap(oia_kmap);
 	do_script(oia_script);
+	do_printer(oia_printer);
 	do_shift(oia_shift);
 	do_typeahead(oia_typeahead);
 	do_compose(oia_compose, oia_compose_char, oia_compose_keytype);
+	do_lu(oia_lu);
 	do_cursor(oia_cursor);
 	do_timing(oia_timing);
 }
@@ -451,6 +468,13 @@ status_half_connect(Boolean ignored unused)
 	status_uncursor_pos();
 }
 
+/* Toggle printer session mode */
+static void
+status_printer(Boolean on)
+{
+	do_printer(oia_printer = on);
+}
+
 /* Lock the keyboard (twait) */
 void
 status_twait(void)
@@ -582,6 +606,18 @@ status_compose(Boolean on, unsigned char c, enum keytype keytype)
 	do_compose(on, c, keytype);
 }
 
+/* Set LU name */
+void
+status_lu(const char *lu)
+{
+	if (lu != NULL) {
+		(void) strncpy(oia_lu, lu, LUCNT);
+		oia_lu[LUCNT] = '\0';
+	} else
+		(void) memset(oia_lu, '\0', sizeof(oia_lu));
+	do_lu(oia_lu);
+}
+
 /* Display timing */
 void
 status_timing(struct timeval *t0, struct timeval *t1)
@@ -1029,6 +1065,12 @@ do_script(Boolean on)
 	status_add(SCRIPT, on ? (*standard_font ? 'S' : CG_S) : nullblank, KT_STD);
 }
 
+static void
+do_printer(Boolean on)
+{
+	status_add(PSESS, on ? (*standard_font ? 'P' : CG_P) : nullblank, KT_STD);
+}
+
 static void
 do_shift(int state)
 {
@@ -1060,6 +1102,19 @@ do_compose(Boolean on, unsigned char c, enum keytype keytype)
 	}
 }
 
+static void
+do_lu(const char *lu)
+{
+	int i;
+
+	for (i = 0; i < LUCNT; i++) {
+		status_add(LU+i,
+		    lu[i]? (*standard_font? lu[i]: asc2cg[(int)lu[i]]):
+			nullblank,
+		    KT_STD);
+	}
+}
+
 /* Timing */
 static void
 do_timing(char *buf)
