@@ -1,10 +1,15 @@
 /*
- * Copyright 1993, 1994, 1995, 1999, 2000, 2001 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001, 2002 by Paul Mattes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
  *  both that copyright notice and this permission notice appear in
  *  supporting documentation.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
@@ -23,7 +28,6 @@
 #endif /*]*/
 #include <errno.h>
 #include <signal.h>
-#include <time.h>
 #include <stdarg.h>
 #include <fcntl.h>
 #include "3270ds.h"
@@ -491,11 +495,27 @@ get_tracef_max(void)
 	}
 }
 
+/* Parse the name '/dev/fd<n>', so we can simulate it. */
+static int
+get_devfd(const char *pathname)
+{
+	unsigned long fd;
+	char *ptr;
+
+	if (strncmp(pathname, "/dev/fd/", 8))
+		return -1;
+	fd = strtoul(pathname + 8, &ptr, 10);
+	if (ptr == pathname + 8 || *ptr != '\0' || fd < 0)
+		return -1;
+	return fd;
+}
+
 /* Callback for "OK" button on trace popup */
 static void
 tracefile_callback(Widget w, XtPointer client_data, XtPointer call_data unused)
 {
 	char *tfn = CN;
+	int devfd = -1;
 #if defined(X3270_DISPLAY) /*[*/
 	int pipefd[2];
 	Boolean just_piped = False;
@@ -511,7 +531,7 @@ tracefile_callback(Widget w, XtPointer client_data, XtPointer call_data unused)
 	tfn = do_subst(tfn, True, True);
 	if (strchr(tfn, '\'') ||
 	    ((int)strlen(tfn) > 0 && tfn[strlen(tfn)-1] == '\\')) {
-		popup_an_error("Illegal file name: %s\n", tfn);
+		popup_an_error("Illegal file name: %s", tfn);
 		Free(tfn);
 		return;
 	}
@@ -524,7 +544,7 @@ tracefile_callback(Widget w, XtPointer client_data, XtPointer call_data unused)
 		tracef = stdout;
 	} else {
 #if defined(X3270_DISPLAY) /*[*/
-		FILE *pipefile;
+		FILE *pipefile = NULL;
 
 		if (!strcmp(tfn, "none") || !tfn[0]) {
 			just_piped = True;
@@ -579,7 +599,10 @@ tracefile_callback(Widget w, XtPointer client_data, XtPointer call_data unused)
 			}
 
 			/* Open and configure the file. */
-			tracef = fopen(tfn, tracef_max? "w+": "a");
+			if ((devfd = get_devfd(tfn)) >= 0)
+				tracef = fdopen(dup(devfd), "a");
+			else
+				tracef = fopen(tfn, tracef_max? "w+": "a");
 			if (tracef == (FILE *)NULL) {
 				popup_an_errno(errno, tfn);
 #if defined(X3270_DISPLAY) /*[*/
@@ -663,7 +686,7 @@ tracefile_on(int reason, enum toggle_type tt)
 		return;
 
 	trace_reason = reason;
-	if (appres.secure) {
+	if (appres.secure && tt != TT_INITIAL) {
 		tracefile_callback((Widget)NULL, "none", PN);
 		return;
 	}
@@ -676,7 +699,7 @@ tracefile_on(int reason, enum toggle_type tt)
 	}
 
 #if defined(X3270_DISPLAY) /*[*/
-	if (tt == TT_INITIAL)
+	if (tt == TT_INITIAL || tt == TT_ACTION)
 #endif /*]*/
 	{
 		tracefile_callback((Widget)NULL, tracefile, PN);
@@ -885,7 +908,7 @@ toggle_screenTrace(struct toggle *t unused, enum toggle_type tt)
 				appres.trace_dir, getpid());
 			tracefile = tracefile_buf;
 		}
-		if (tt == TT_INITIAL) {
+		if (tt == TT_INITIAL || tt == TT_ACTION) {
 			(void) screentrace_cb(NewString(tracefile));
 			return;
 		}
