@@ -1,16 +1,21 @@
 /*
- * Copyright 1993, 1994, 1995, 1999, 2000, 2001 by Paul Mattes.
+ * Copyright 1993, 1994, 1995, 1999, 2000, 2001, 2002 by Paul Mattes.
  * Parts Copyright 1990 by Jeff Sparkes.
  *  Permission to use, copy, modify, and distribute this software and its
  *  documentation for any purpose and without fee is hereby granted,
  *  provided that the above copyright notice appear in all copies and that
  *  both that copyright notice and this permission notice appear in
  *  supporting documentation.
+ *
+ * x3270, c3270, s3270 and tcl3270 are distributed in the hope that they will
+ * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the file LICENSE
+ * for more details.
  */
 
 /*
  *	util.c
- *		Utility functions for x3270
+ *		Utility functions for x3270/c3270/s3270/tcl3270
  */
 
 #include "globals.h"
@@ -125,7 +130,7 @@ scatv(const char *s, char *buf, size_t len)
 	char *dst = buf;
 
 	while ((c = *s++) && len > 0) {
-		char cbuf[4];
+		char cbuf[5];
 		char *t = cbuf;
 
 		/* Expand this character. */
@@ -547,6 +552,39 @@ get_fresource(const char *fmt, ...)
 	return r;
 }
 
+/*
+ * A version of get_resource that tries to find a host-specific definition
+ * first, then finds the non-host version.
+ */
+char *
+get_host_fresource(const char *fmt, ...)
+{
+	va_list args;
+	char *name;
+	char *r;
+
+	/* Format what they want. */
+	va_start(args, fmt);
+	name = xs_vsprintf(fmt, args);
+	va_end(args);
+
+	/* If we're connected, try by-host first. */
+	if (PCONNECTED) {
+		char *name_with_host;
+
+		name_with_host = xs_buffer("host.%s.%s", reconnect_host, name);
+		r = get_resource(name_with_host);
+		Free(name_with_host);
+		if (r != CN)
+			return r;
+	}
+
+	/* Try without. */
+	r = get_resource(name);
+	Free(name);
+	return r;
+}
+
 #if defined(X3270_DISPLAY) /*[*/
 
 /* Glue between x3270 and the X libraries. */
